#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TinyLibrary.hpp"
#include "Player.hpp"
#include "BgMap.hpp"
#include "Global.hpp"
#include "Resource.hpp"
#include "SoundPlayer.hpp"

void MainLoop_Setup(void)
{
	Global::Initialize();
	Screen::Initialize(8);
	Controller::Initialize();
	SoundPlayer::Initialize(11025);
	Global& global = Global::GetInstance();
	global.phase = Global::PHASE_LOGO;
	global.back_color = 0;
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	SoundData* sound_data = GetSoundData(SOUND_BGM);
	global.bgm_handle = sound_player.Append(0, "SuperOteme/bgm.raw", true);
	if(global.bgm_handle == -1)
	{
		global.bgm_handle = sound_player.Append(0, sound_data->buffer, sound_data->size, true);
	}
	sound_data = GetSoundData(SOUND_PIYO);
	global.piyo_handle = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_JUMP);
	global.jump_handle = sound_player.Append(2, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_KNOCKDOWN);
	global.knockdown_handle = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
}

void MainLoop_Finalize(void)
{
	SoundPlayer::Finalize();
	Controller::Finalize();
	Screen::Finalize();
	Global::Finalize();
}

void MainLoop_Loop(void)
{
	ImageData* image_data;
	Global& global = Global::GetInstance();
	Screen& screen = Screen::GetInstance();
	Controller& controller = Controller::GetInstance();
	controller.Update();
	screen.Clear(global.back_color);
	screen.DrawBegin();
	unsigned int button = controller.GetButton();
	bool status;
	switch(global.phase)
	{
	case Global::PHASE_LOGO:
		image_data = GetImageData(LOGO);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		global.back_color = 0;
		if((global.button_before == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_TITLE;
		}
		++ global.count;
		if(global.count > 30 * 5)
		{
			global.phase = Global::PHASE_TITLE;
			global.count = 0;
		}
		break;
	case Global::PHASE_TITLE:
		image_data = GetImageData(TITLE);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		global.back_color = 0;
		if((global.button_before == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_TITLE2;
		}
		++ global.count;
		if(global.count > 30 * 5)
		{
			global.phase = Global::PHASE_TITLE2;
			global.count = 0;
		}
		break;
	case Global::PHASE_TITLE2:
		image_data = GetImageData(TITLE2);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(0, 0, "%8d", global.score);
		screen.DrawNumberFont(64, 0, "%8d", global.highscore);
		global.back_color = 0;
		if((global.button_before == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_INITIALIZE_GAME;
		}
		++ global.count;
		if(global.count > 30 * 10)
		{
			global.phase = Global::PHASE_LOGO;
			global.count = 0;
		}
		if((global.count % 16) < 8)
		{
			image_data = GetImageData(METEOR_1);
			screen.DrawSprite(image_data->buffer, 32, 44, image_data->width, image_data->height, 0);
		}
		else
		{
			image_data = GetImageData(METEOR_2);
			screen.DrawSprite(image_data->buffer, 32, 44, image_data->width, image_data->height, 0);
		}
		break;
	case Global::PHASE_INITIALIZE_GAME:
		global.back_color = 0;
		global.game.Initialize(1);
		global.phase = Global::PHASE_GAME;
		break;
	case Global::PHASE_GAME:
		status = global.game.Update();
		global.game.Draw();
		global.back_color = global.game.GetBackColor();
		if(status == true)
		{
			global.game.Finalize();
			global.phase = Global::PHASE_LOGO;
			global.count = 0;
		}
		break;
	}
	screen.DrawEnd();
	global.button_before = button;
}

void MainLoop_NoWaitLoop(void)
{
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.Update();
}
